import { Module } from '@nestjs/common';
import { ConfigModule, ConfigService } from '@nestjs/config';
// import { HealthModule } from './modules/health/health.module';
import { AdModule } from './modules/ad/ad.module';
import {
  MessageBrokerModule,
  MessageBrokerModuleOptions,
} from '@mobicoop/message-broker-module';
import {
  ConfigurationModule,
  ConfigurationModuleOptions,
} from '@mobicoop/configuration-module';
import { EventEmitterModule } from '@nestjs/event-emitter';
import { RequestContextModule } from 'nestjs-request-context';
import { HealthModule } from '@modules/health/health.module';

@Module({
  imports: [
    ConfigModule.forRoot({ isGlobal: true }),
    EventEmitterModule.forRoot(),
    RequestContextModule,
    MessageBrokerModule.forRootAsync({
      imports: [ConfigModule],
      inject: [ConfigService],
      useFactory: async (
        configService: ConfigService,
      ): Promise<MessageBrokerModuleOptions> => ({
        uri: configService.get<string>('MESSAGE_BROKER_URI'),
        exchange: configService.get<string>('MESSAGE_BROKER_EXCHANGE'),
        name: 'ad',
      }),
    }),
    ConfigurationModule.forRootAsync({
      imports: [ConfigModule],
      inject: [ConfigService],
      useFactory: async (
        configService: ConfigService,
      ): Promise<ConfigurationModuleOptions> => ({
        domain: configService.get<string>('SERVICE_CONFIGURATION_DOMAIN'),
        messageBroker: {
          uri: configService.get<string>('MESSAGE_BROKER_URI'),
          exchange: configService.get<string>('MESSAGE_BROKER_EXCHANGE'),
        },
        redis: {
          host: configService.get<string>('REDIS_HOST'),
          password: configService.get<string>('REDIS_PASSWORD'),
          port: configService.get<number>('REDIS_PORT'),
        },
        setConfigurationBrokerQueue: 'ad-configuration-create-update',
        deleteConfigurationQueue: 'ad-configuration-delete',
        propagateConfigurationQueue: 'ad-configuration-propagate',
      }),
    }),
    HealthModule,
    AdModule,
  ],
  controllers: [],
  providers: [],
})
export class AppModule {}
