import { Controller, UsePipes } from '@nestjs/common';
import { CommandBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { CreateAdRequestDto } from './dtos/create-ad.request.dto';
import { CreateAdCommand } from '../../core/commands/create-ad/create-ad.command';
import { AggregateID } from '@libs/ddd';
import { AdAlreadyExistsException } from '../../core/ad.errors';
import { IdResponse } from '@libs/api/id.response.dto';
import { RpcExceptionCode } from '@libs/exceptions/rpc-exception.codes.enum';
import { RpcValidationPipe } from '@libs/utils/pipes/rpc.validation-pipe';

@UsePipes(
  new RpcValidationPipe({
    whitelist: false,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class CreateAdGrpcController {
  constructor(private readonly commandBus: CommandBus) {}

  @GrpcMethod('AdsService', 'Create')
  async create(data: CreateAdRequestDto): Promise<IdResponse> {
    try {
      const aggregateID: AggregateID = await this.commandBus.execute(
        new CreateAdCommand(data),
      );
      return new IdResponse(aggregateID);
    } catch (error: any) {
      if (error instanceof AdAlreadyExistsException)
        throw new RpcException({
          code: RpcExceptionCode.ALREADY_EXISTS,
          message: error.message,
        });
      throw new RpcException({
        code: RpcExceptionCode.UNKNOWN,
        message: error.message,
      });
    }
  }
}
