import { AD_REPOSITORY } from '@modules/ad/ad.di-tokens';
import { AdEntity } from '@modules/ad/core/ad.entity';
import {
  CreateAdProps,
  DefaultAdProps,
  Frequency,
} from '@modules/ad/core/ad.types';
import { FindAdByIdQuery } from '@modules/ad/core/queries/find-ad-by-id/find-ad-by-id.query';
import { FindAdByIdQueryHandler } from '@modules/ad/core/queries/find-ad-by-id/find-ad-by-id.query-handler';
import { MarginDurationsProps } from '@modules/ad/core/value-objects/margin-durations.value-object';
import { WaypointProps } from '@modules/ad/core/value-objects/waypoint.value-object';
import { Test, TestingModule } from '@nestjs/testing';

const originWaypointProps: WaypointProps = {
  position: 0,
  address: {
    houseNumber: '5',
    street: 'Avenue Foch',
    locality: 'Nancy',
    postalCode: '54000',
    country: 'France',
    coordinates: {
      lon: 48.689445,
      lat: 6.17651,
    },
  },
};
const destinationWaypointProps: WaypointProps = {
  position: 1,
  address: {
    locality: 'Paris',
    postalCode: '75000',
    country: 'France',
    coordinates: {
      lon: 48.8566,
      lat: 2.3522,
    },
  },
};
const marginDurationsProps: MarginDurationsProps = {
  mon: 600,
  tue: 600,
  wed: 600,
  thu: 600,
  fri: 600,
  sat: 600,
  sun: 600,
};
const baseCreateAdProps = {
  userId: 'e8fe64b1-4c33-49e1-9f69-4db48b21df36',
  seatsProposed: 3,
  seatsRequested: 1,
  strict: false,
  waypoints: [originWaypointProps, destinationWaypointProps],
};
const punctualCreateAdProps = {
  fromDate: '2023-06-22',
  toDate: '2023-06-22',
  schedule: {
    wed: '08:30',
  },
  frequency: Frequency.PUNCTUAL,
};
const punctualPassengerCreateAdProps: CreateAdProps = {
  ...baseCreateAdProps,
  ...punctualCreateAdProps,
  marginDurations: marginDurationsProps,
  driver: false,
  passenger: true,
};

const defaultAdProps: DefaultAdProps = {
  driver: false,
  passenger: true,
  marginDurations: {
    mon: 900,
    tue: 900,
    wed: 900,
    thu: 900,
    fri: 900,
    sat: 900,
    sun: 900,
  },
  seatsProposed: 3,
  seatsRequested: 1,
  strict: false,
};

const ad: AdEntity = AdEntity.create(
  punctualPassengerCreateAdProps,
  defaultAdProps,
);

const mockAdRepository = {
  findOneById: jest.fn().mockImplementation(() => ad),
};

describe('find-ad-by-id.query-handler', () => {
  let findAdByIdQueryHandler: FindAdByIdQueryHandler;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        {
          provide: AD_REPOSITORY,
          useValue: mockAdRepository,
        },
        FindAdByIdQueryHandler,
      ],
    }).compile();

    findAdByIdQueryHandler = module.get<FindAdByIdQueryHandler>(
      FindAdByIdQueryHandler,
    );
  });

  it('should be defined', () => {
    expect(findAdByIdQueryHandler).toBeDefined();
  });

  describe('execution', () => {
    it('should return an ad', async () => {
      const findAdbyIdQuery = new FindAdByIdQuery(
        'dd264806-13b4-4226-9b18-87adf0ad5dd1',
      );
      const ad: AdEntity = await findAdByIdQueryHandler.execute(
        findAdbyIdQuery,
      );
      expect(ad.getProps().fromDate).toBe('2023-06-22');
    });
  });
});
