import { Inject, Injectable } from '@nestjs/common';
import {
  HealthCheckError,
  HealthCheckResult,
  HealthIndicator,
  HealthIndicatorResult,
} from '@nestjs/terminus';
import { CheckRepositoryPort } from '../ports/check-repository.port';
import { AD_REPOSITORY } from '@modules/health/health.di-tokens';
import { AdRepositoryPort } from '@modules/ad/core/ports/ad.repository.port';
import { MESSAGE_PUBLISHER } from '@src/app.constants';
import { LOGGING_AD_HEALTH_CRIT } from '@modules/health/health.constants';
import { MessagePublisherPort } from '@libs/ports/message-publisher.port';

@Injectable()
export class RepositoriesHealthIndicatorUseCase extends HealthIndicator {
  private checkRepositories: CheckRepositoryPort[];
  constructor(
    @Inject(AD_REPOSITORY)
    private readonly adRepository: AdRepositoryPort,
    @Inject(MESSAGE_PUBLISHER)
    private readonly messagePublisher: MessagePublisherPort,
  ) {
    super();
    this.checkRepositories = [adRepository];
  }
  isHealthy = async (key: string): Promise<HealthIndicatorResult> => {
    try {
      await Promise.all(
        this.checkRepositories.map(
          async (checkRepository: CheckRepositoryPort) => {
            await checkRepository.healthCheck();
          },
        ),
      );
      return this.getStatus(key, true);
    } catch (error) {
      const healthCheckResult: HealthCheckResult = error;
      this.messagePublisher.publish(
        LOGGING_AD_HEALTH_CRIT,
        JSON.stringify(healthCheckResult.error),
      );
      throw new HealthCheckError('Repository', {
        repository: error.message,
      });
    }
  };
}
