import { Module } from '@nestjs/common';
import { HealthHttpController } from './interface/http-controllers/health.http.controller';
import { TerminusModule } from '@nestjs/terminus';
import { MESSAGE_BROKER_PUBLISHER, MESSAGE_PUBLISHER } from 'src/app.constants';
import { MessageBrokerPublisher } from '@mobicoop/message-broker-module';
import { MessagePublisher } from './infrastructure/message-publisher';
import { RepositoriesHealthIndicatorUseCase } from './core/usecases/repositories.health-indicator.usecase';
import { AdRepository } from '../ad/infrastructure/ad.repository';
import { AD_REPOSITORY } from './health.di-tokens';
import { HealthGrpcController } from './interface/grpc-controllers/health.grpc.controller';
import { AdModule } from '@modules/ad/ad.module';

@Module({
  imports: [TerminusModule, AdModule],
  controllers: [HealthGrpcController, HealthHttpController],
  providers: [
    RepositoriesHealthIndicatorUseCase,
    {
      provide: AD_REPOSITORY,
      useClass: AdRepository,
    },
    {
      provide: MESSAGE_BROKER_PUBLISHER,
      useClass: MessageBrokerPublisher,
    },
    {
      provide: MESSAGE_PUBLISHER,
      useClass: MessagePublisher,
    },
  ],
})
export class HealthModule {}
